@@ -22,14 +22,13 @@ module Test.MockCat.TH
 where
 
 import Control.Monad (guard, unless)
-import Control.Monad.State (get, modify)
+import Control.Monad.IO.Class (MonadIO, liftIO)
 import Control.Monad.Trans.Class (lift)
 import Data.Data (Proxy (..))
 import Data.Function ((&))
 import Data.List (elemIndex, find, nub)
 import Data.Maybe (fromMaybe, isJust)
 import Data.Text (pack, splitOn, unpack)
-import GHC.IO (unsafePerformIO)
 import GHC.TypeLits (KnownSymbol, symbolVal)
 import Language.Haskell.TH
   ( Cxt,
@@ -351,18 +350,24 @@ hasMonadInVarInfo (VarName2ClassNames _ classNames) = ''Monad `elem` classNames
 
 createCxt :: [Pred] -> MockType -> Name -> Name -> [TyVarBndr a] -> [VarAppliedType] -> Q [Pred]
 createCxt cxt mockType className monadVarName tyVars varAppliedTypes = do
-  newCxt <- mapM (createPred monadVarName) cxt
+  newCxtRaw <- mapM (createPred monadVarName) cxt
 
-  monadAppT <- appT (conT ''Monad) (varT monadVarName)
+  let isRedundantMonad (AppT (ConT m) (VarT v)) = m == ''Monad && v == monadVarName
+      isRedundantMonad _ = False
+      newCxt = filter (not . isRedundantMonad) newCxtRaw
 
-  let hasMonad = P.any (\(ClassName2VarNames c _) -> c == ''Monad) $ toClassInfos newCxt
+  monadIOAppT <- appT (conT ''MonadIO) (varT monadVarName)
+
+  let classInfos = toClassInfos newCxt
+      hasMonadIO = P.any (\(ClassName2VarNames c _) -> c == ''MonadIO) classInfos
+      addedMonads = [monadIOAppT | not hasMonadIO]
 
   pure $ case mockType of
-    Total -> newCxt ++ ([monadAppT | not hasMonad])
+    Total -> newCxt ++ addedMonads
     Partial -> do
       let classAppT = constructClassAppT className $ toVarTs tyVars
           varAppliedClassAppT = updateType classAppT varAppliedTypes
-      newCxt ++ ([monadAppT | not hasMonad]) ++ [varAppliedClassAppT]
+      newCxt ++ addedMonads ++ [varAppliedClassAppT]
 
 toVarTs :: [TyVarBndr a] -> [Type]
 toVarTs tyVars = VarT <$> getTypeVarNames tyVars
@@ -417,7 +422,7 @@ generateInstanceMockFnBody fnNameStr args r options = do
     else [| lift $(varE r) |]
   [|
     MockT $ do
-      defs <- get
+      defs <- getDefinitions
       let mock =
             defs
               & findParam (Proxy :: Proxy $(litT (strTyLit fnNameStr)))
@@ -433,7 +438,7 @@ generateInstanceRealFnBody fnName fnNameStr args r options = do
     else [| lift $(varE r) |]
   [|
     MockT $ do
-      defs <- get
+      defs <- getDefinitions
       case findParam (Proxy :: Proxy $(litT (strTyLit fnNameStr))) defs of
         Just mock -> do
           let $(bangP $ varP r) = $(generateStubFn args [|mock|])
@@ -482,7 +487,7 @@ doCreateMockFnDecs funNameStr mockFunName params funType monadVarName updatedTyp
     sigD
       mockFunName
       [t|
-        (MockBuilder $(varT params) ($(pure funType)) ($(pure verifyParams)), Monad $(varT monadVarName)) =>
+        (MockBuilder $(varT params) ($(pure funType)) ($(pure verifyParams)), MonadIO $(varT monadVarName)) =>
         $(varT params) ->
         MockT $(varT monadVarName) ()
         |]
@@ -496,7 +501,7 @@ doCreateMockFnDecs funNameStr mockFunName params funType monadVarName updatedTyp
 
 doCreateConstantMockFnDecs :: (Quote m) => String -> Name -> Type -> Name -> m [Dec]
 doCreateConstantMockFnDecs funNameStr mockFunName ty monadVarName = do
-  newFunSig <- sigD mockFunName [t|(Monad $(varT monadVarName)) => $(pure ty) -> MockT $(varT monadVarName) ()|]
+  newFunSig <- sigD mockFunName [t|(MonadIO $(varT monadVarName)) => $(pure ty) -> MockT $(varT monadVarName) ()|]
   createMockFn <- [|createNamedConstantMock|]
   mockBody <- createMockBody funNameStr createMockFn
   newFun <- funD mockFunName [clause [varP $ mkName "p"] (normalB (pure mockBody)) []]
@@ -510,7 +515,7 @@ doCreateEmptyVerifyParamMockFnDecs funNameStr mockFunName params funType monadVa
     sigD
       mockFunName
       [t|
-        (MockBuilder $(varT params) ($(pure funType)) (), Monad $(varT monadVarName)) =>
+        (MockBuilder $(varT params) ($(pure funType)) (), MonadIO $(varT monadVarName)) =>
         $(varT params) ->
         MockT $(varT monadVarName) ()
         |]
@@ -525,14 +530,13 @@ doCreateEmptyVerifyParamMockFnDecs funNameStr mockFunName params funType monadVa
 createMockBody :: (Quote m) => String -> Exp -> m Exp
 createMockBody funNameStr createMockFn =
   [|
-    MockT $
-      modify
-        ( ++
-            [ Definition
-                (Proxy :: Proxy $(litT (strTyLit funNameStr)))
-                (unsafePerformIO $ $(pure createMockFn) $(litE (stringL funNameStr)) p)
-                shouldApplyToAnything
-            ]
+    MockT $ do
+      mockInstance <- liftIO $ $(pure createMockFn) $(litE (stringL funNameStr)) p
+      addDefinition
+        ( Definition
+            (Proxy :: Proxy $(litT (strTyLit funNameStr)))
+            mockInstance
+            shouldApplyToAnything
         )
     |]
 
